﻿//////////////////////////////////////////////
// TestClass.h
// 
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward decl -----------------------------

namespace nkDebug
{
	class TestRunParameters ;

	struct ClassInitFunctionDescriptor ;
	struct ClassShutdownFunctionDescriptor ;
}

/// Includes ---------------------------------

// nkDebug
#include "../Dll/DllDefines.h"

#include "FunctionSet.h"
#include "UnitTester.h"

/// Class ------------------------------------

namespace nkDebug
{
	class DLL_NKDEBUG_EXPORT TestClass
	{
		public :

			// Attributes
			// Functions
			FunctionSet _testFunctions ;
			ClassInitFunctionDescriptor* _classInitFunction ;
			ClassShutdownFunctionDescriptor* _classShutdownFunction ;
			ClassInitFunctionDescriptor* _testFunctionInitFunction ;
			ClassShutdownFunctionDescriptor* _testFunctionShutdownFunction ;

			// Tracking name
			const char* _name ;

		public :

			// Constructor, destructor
			TestClass () ;
			TestClass (const nkLog::StringView& name) ;
			virtual ~TestClass () ;

			// Execution
			void execute (const TestRunParameters& parameters) const ;

			// Registering
			bool setClassInitFunction (const std::function<bool ()>& func) ;
			bool setClassShutdownFunction (const std::function<void ()>& func) ;
			bool setTestFunctionInitFunction (const std::function<bool ()>& func) ;
			bool setTestFunctionShutdownFunction (const std::function<void ()>& func) ;
	} ;

	static const char NK_DEBUG_STATIC_EMPTY_CHAR [] = "" ;

	template <typename T, const char* T_NAME>
	class RegisteredTestClass : public TestClass
	{
		public :

			// Functions
			// Constructor
			RegisteredTestClass ()
			:	TestClass (T_NAME)
			{
				// Nothing to do
			}

			// Factory
			static TestClass* factoryFunc ()
			{
				return new T () ;
			}

		public :

			// Attributes
			// Registered class pointer
			inline static TestClass* _registered = UnitTester::registerTestClass(&factoryFunc) ;
	} ;
}

/// Utility macros ---------------------------

#define NK_TEST_CLASS(name)	\
	static const char NK_DEBUG_STATIC_CHAR_##name [] = #name ; \
	class name : public nkDebug::RegisteredTestClass<name, NK_DEBUG_STATIC_CHAR_##name>

#define NK_TEST_FUNCTION(name) \
	static bool caller_##name () \
	{ \
		return name() ; \
	} \
	\
	inline static const std::function<bool ()>& registered_##name = _registered->_testFunctions.registerFunction(#name, &caller_##name) ; \
	\
	static bool name () \

#define NK_TEST_CLASS_INIT \
	static bool classInitFunctionCaller () \
	{ \
		return classInitFunction() ; \
	} \
	\
	inline static const bool classInitRegistered = _registered->setClassInitFunction(classInitFunctionCaller) ; \
	\
	static bool classInitFunction () \

#define NK_TEST_CLASS_SHUTDOWN \
	static void classShutdownFunctionCaller () \
	{ \
		classShutdownFunction() ; \
	} \
	\
	inline static const bool classShutdownRegistered = _registered->setClassShutdownFunction(classShutdownFunctionCaller) ; \
	\
	static void classShutdownFunction () \

#define NK_TEST_FUNCTION_INIT \
	static bool testFunctionInitFunctionCaller () \
	{ \
		return testFunctionInitFunction() ; \
	} \
	\
	inline static const bool testFunctionInitRegistered = _registered->setTestFunctionInitFunction(testFunctionInitFunctionCaller) ; \
	\
	static bool testFunctionInitFunction () \

#define NK_TEST_FUNCTION_SHUTDOWN \
	static void testFunctionShutdownFunctionCaller () \
	{ \
		testFunctionShutdownFunction() ; \
	} \
	\
	inline static const bool testFunctionShutdownRegistered = _registered->setTestFunctionShutdownFunction(testFunctionShutdownFunctionCaller) ; \
	\
	static void testFunctionShutdownFunction () \

#define NK_TEST_VALIDATE return true ;

#define NK_TEST_REJECT return false ;

#define NK_TEST_CHECK(x) if (!x) return false ;