﻿//////////////////////////////////////////////
// InstancingPass.h
//
//////////////////////////////////////////////

/// Defines / Macros -------------------------

#pragma once

/// Forward declarations ---------------------

namespace nkGraphics
{
	class BlendState ;
	class Camera ;
	class DepthStencilState ;
	class Mesh ;
	class RasterState ;
	class Shader ;
}

/// Includes ---------------------------------

// nkGraphics
#include "Pass.h"

#include "../Dll/DllDefines.h"

/// Class ------------------------------------

namespace nkGraphics
{
	class DLL_GRAPHICS_EXPORT InstancingPass : public Pass
	{
		protected :

			// Le mesh qu'on va utiliser pour faire le rendu
			Mesh* _toRender ;
			
			// Le nombre d'instances à dessiner
			unsigned int _instanceNumber ;

			// Le shader à utiliser
			Shader* _shader ;

			// Camera si besoin est
			Camera* _renderCam ;
			Camera* _cullingCam ;
			Camera* _lodCam ;

			// States
			BlendState* _blendState ;
			RasterState* _rasterState ;
			DepthStencilState* _depthState ;

		protected :

			// Constructor, destructor
			InstancingPass () ;
			virtual ~InstancingPass () ;

		public :
	
			// Getters
			Mesh* getToRender () const ;
			unsigned int getInstanceNumber () const ;
			Shader* getShader () const ;
			Camera* getRenderCam () const ;
			Camera* getCullingCam () const ;
			Camera* getLodCam () const ;
			BlendState* getBlendState () const ;
			RasterState* getRasterState () const ;
			DepthStencilState* getDepthStencilState () const ;

			// Setters
			virtual void setToRender (Mesh* object) ;
			void setInstanceNumber (unsigned int value) ;
			virtual void setShader (Shader* value) ;
			void setRenderCam (Camera* value) ;
			void setCullingCam (Camera* value) ;
			void setLodCam (Camera* value) ;
			virtual void setBlendState (BlendState* holder) ;
			virtual void setRasterState (RasterState* holder) ;
			virtual void setDepthStencilState (DepthStencilState* holder) ;

			// Exports
			virtual void exportClassToTree (nkExport::Node* rootNode) override ;
			virtual void importClassFromTree (nkExport::Node* rootNode) override ;
	} ;
}